@@ -1037,16 +1037,16 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
     out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_PROGRESSIVE;
 
   switch (context->chroma_sample_location) {
-    case 1:
+    case AVCHROMA_LOC_LEFT:
       out_info->chroma_site = GST_VIDEO_CHROMA_SITE_MPEG2;
       break;
-    case 2:
+    case AVCHROMA_LOC_CENTER:
       out_info->chroma_site = GST_VIDEO_CHROMA_SITE_JPEG;
       break;
-    case 3:
+    case AVCHROMA_LOC_TOPLEFT:
       out_info->chroma_site = GST_VIDEO_CHROMA_SITE_DV;
       break;
-    case 4:
+    case AVCHROMA_LOC_TOP:
       out_info->chroma_site = GST_VIDEO_CHROMA_SITE_V_COSITED;
       break;
     default:
@@ -2044,8 +2044,11 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
         || in_plugin->id == AV_CODEC_ID_Y41P
         || in_plugin->id == AV_CODEC_ID_012V
         || in_plugin->id == AV_CODEC_ID_YUV4
-        || in_plugin->id == AV_CODEC_ID_ZLIB
-        || in_plugin->id == AV_CODEC_ID_WRAPPED_AVFRAME) {
+#if AV_VERSION_INT (LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO) >= \
+        AV_VERSION_INT (57,4,0)
+        || in_plugin->id == AV_CODEC_ID_WRAPPED_AVFRAME
+#endif
+        || in_plugin->id == AV_CODEC_ID_ZLIB) {
       goto next;
     }
 
@@ -2082,6 +2085,13 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
       goto next;
     }
 
+    if (g_str_has_suffix (in_plugin->name, "_qsv")) {
+      GST_DEBUG
+          ("Ignoring qsv decoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
     GST_DEBUG ("Trying plugin %s [%s]", in_plugin->name, in_plugin->long_name);
 
     /* no codecs for which we're GUARANTEED to have better alternatives */
